@@ -1,5 +1,3 @@
-#	$Header: /var/cvs/sources/Funlog/lib/Log/Funlog.pm,v 1.48 2005/02/17 11:24:25 gab Exp $
-
 =head1 NAME
 
 Log::Funlog - Log module with fun inside!
@@ -12,7 +10,7 @@ Log::Funlog - Log module with fun inside!
 	...
  );
 
- my_sub(priority,string,string, ... );
+ [$string=]my_sub($priority [,$string | @array [,$string | @array [, ... ] ]] );
 
 =head1 DESCRIPTION
 
@@ -24,20 +22,18 @@ Just initialize the module, then use is as if it was an ordinary function!
 
 When you want to log something, just write:
 
- your-sub-log(priority,"what"," I ","wanna log")
+ your-sub-log(priority,"what"," I ","wanna log is: ",@an_array)
 
 then the module will analyse if the priority if higher enough (seeing L</verbose> option). If yes, your log will be written with the format you decided on STDERR (default) or a file.
 
 As more, the module can write funny things to your logs, if you want ;) It can be very verbose, or just ... shy :)
 
-L<Log::Funlog|Log::Funlog> may export an 'error' function: it logs your message with a priority of 1 and with an specific (parametrable) string. You can use it when you want to highlight error messages in your logs.
+L<Log::Funlog|Log::Funlog> may export an 'error' function: it logs your message with a priority of 1 and with an specific (parametrable) string. You can use it when you want to highlight error messages in your logsi with a pattern.
 
-Parameters are: L</header>, L</error_header>, L</cosmetic>, L</verbose>, L</file>, L</daemon>, L</fun> and L</caller>
+Parameters are: L<header>, L<error_header>, L<cosmetic>, L<verbose>, L<file>, L<daemon>, L<fun>, L<colors>, L<splash> and L<caller>
 
 L</verbose> is mandatory.
 
-B<NOTE NOTE NOTE>: Interface (L</header>) is subject to change!
-
 =head2 MANDATORY OPTION
 
 =over
@@ -77,6 +73,83 @@ See L</EXAMPLE>
 
 =over
 
+=item B<caller>
+
+'all' if you want the stack of subs.
+
+'last' if you want the last call.
+
+If you specify a number B<n>, it will print the B<n> last calls (yes, if you specify '1', it is equivalent to 'last')
+
+If this number is negative, it will print the B<n> first calls.
+
+Of course, nothing will happen if no L</header> is specified, nor %ss in the L</header> ...
+
+=item B<colors>
+
+Put colors in the logs :)
+
+If you just put '1', it will use default colors:
+
+ colors => '1',
+
+If you want to override default colors, specify a hash containing item => color
+
+ colors => {'prog' => 'white', 'date' => 'yellow' },
+
+Items are:
+
+	caller: for the stack of calls,
+	prog: for the name of the program,
+	date: for the current date,
+	level: for the log level,
+	msg: for the log message
+
+Colors are:
+	black, red, green, yellow, blue, magenta, cyan, white and none
+
+=item B<cosmetic>
+
+An alphanumeric char to indicate the log level in your logs.
+
+There will be as many as these chars as the log level of the string being logged. See L</EXAMPLE>
+
+Should be something like 'x', or '*', or '!', or any printable single character.
+
+=item B<daemon>
+
+1 if the script should be a daemon. (default is 0: not a daemon)
+
+When B<daemon>=1, L</Log::Funlog> write to L</file> instead of B<STDERR>
+
+If you specify B<daemon>, you must specify L</file>
+
+The common way to do is the same that with L</verbose>: with Getopt
+
+=item B<file>
+
+File to write logs to.
+
+MUST be specified if you specify L</daemon>
+
+File is opened when initializing, and never closed by the module. That is mainly to avoid open and close the file each time you log something and then increase speed.
+
+Side effect is that if you tail -f the log file, you won't see them in real time.
+
+=item B<error_header>
+
+Header you want to see in the logs when you call the B<error> function (if you import it, of course)
+
+Default is '## Oops! ##'.
+
+=item B<fun>
+
+Probability of fun in your logs.
+
+Should be: 0<fun<=100
+
+It use Log::Funlog::Lang
+
 =item B<header>
 
 Pattern specifying the header of your logs.
@@ -123,84 +196,9 @@ won't print anything if you log from outside a sub. Both will have the same effe
 You should probably always write things like:
  ' -{((<%dd>))}-<%pp>- -<(%ll)>- '
 
+=item B<splash>
 
-I<NOTE NOTE NOTE>: The fields are subject to change!
-
-=item B<colors>
-
-Put colors in the logs :)
-
-If you just put '1', it will use default colors:
-
- colors => '1',
-
-If you want to override default colors, specify a hash containing item => color
-
- colors => {'prog' => 'white', 'date' => 'yellow' },
-
-Items are:
-	caller: for the stack of calls,
-	prog: for the name of the program,
-	date: for the current date,
-	level: for the log level,
-	msg: for the log message
-
-Colors are:
-	black, red, green, yellow, blue, magenta, cyan, white and none
-
-=item B<daemon>
-
-1 if the script should be a daemon. (default is 0: not a daemon)
-
-When B<daemon>=1, L<Log::Funlog|Log::Funlog> write to L</file> instead of B<STDERR>
-
-If you specify B<daemon>, you must specify L</file>
-
-The common way to do is the same that with L</verbose>: with Getopt
-
-=item B<file>
-
-File to write logs to.
-
-MUST be specified if you specify L</daemon>
-
-File is opened when initializing, and never closed by the module. That is mainly to avoid open and close the file each time you log something and then increase speed.
-
-Side effect is that if you tail -f the log file, you won't see them in real time.
-
-=item B<cosmetic>
-
-An alphanumeric char to indicate the log level in your logs.
-
-There will be as many as these chars as the log level of the string being logged. See L</EXAMPLE>
-
-Should be something like 'x', or '*', or '!', but actually no test are performed to verify that there is only one caracter...
-
-=item B<error_header>
-
-Header you want to see in the logs when you call the B<error> function (if you import it, of course)
-
-Default is '## Oops! ##'.
-
-=item B<fun>=n%
-
-Probs of fun in your logs.
-
-Should be: 0<fun<=100
-
-See the sources of L<Log::Funlog|Log::Funlog> if you want to change the sentences
-
-=item B<caller>
-
-'all' if you want the stack of subs.
-
-'last' if you want the last call.
-
-If you specify a number B<n>, it will print the B<n> last calls (yes, if you specify '1', it is equivalent to 'last')
-
-If this number is negative, it will print the B<n> first calls.
-
-Of course, nothing will happen if no L</header> is specified, nor %ss in the L</header> ...
+1 if you want a 'splash log'
 
 =back
 
@@ -234,16 +232,19 @@ Here is an example with almost all of the options enabled:
 
  :wq
 
- $ ./gna.pl
- Wed Sep 22 18:50:34 2004 [gna.pl] [x    ] I'm logged...
- Wed Sep 22 18:50:34 2004 [gna.pl] [xxx  ] Me too...
- Wed Sep 22 18:50:34 2004 [gna.pl] [x    ] Onetwo1C++
- Wed Sep 22 18:50:34 2004 [gna.pl] [x    ] Groumpf...  oups!
- Wed Sep 22 18:50:34 2004 [gna.pl] [x    ] Groumpf...  Zut
-
+ $ perl gna.pl
+ Tue Jul 26 15:39:41 2005 [gna.pl] [x    ]  I'm logged...
+ Tue Jul 26 15:39:41 2005 [gna.pl] [xxx  ]  Me too...
+ Tue Jul 26 15:39:41 2005 [gna.pl] [x    ] {ze_sub} Onetwo1C++
+ Tue Jul 26 15:39:41 2005 [gna.pl] [x    ] {ze_sub} Groumpf...  oups!
+ Tue Jul 26 15:39:41 2005 [gna.pl] [x    ]  Groumpf...  Zut
 
 =head1 BUGS
 
+=over
+
+=item 1-
+
 This:
 
  header => '-(%dd)--( %p)><(p )-( %l)-<>-(l %s)<>(s '
@@ -256,9 +257,33 @@ Workaround is:
 
 And this kind of workaround work for everything but %ss, as it is not calculated during initialization.
 
+=item 2-
+
+ *Log=Log::Funlog->new(
+  colors => 1,
+  colors => {
+	 date => 'white'
+  }
+ )
+
+Is not the same as:
+
+ *Log=Log::Funlog->new(
+  colors => {
+	 date => 'white'
+  },
+  colors => 1,
+ )
+
+First case will do what you expect, second case will put default colors.
+
+To avoid that, specify EITHER colors => 1 OR colors => {<something>}
+
+=back
+
 =head1 DEPENDENCIES
 
-Log::Funlog::Lang provide the funny messages.
+Log::Funlog::Lang > 0.3 : provide the funny messages.
 
 =head1 DISCUSSION
 
@@ -298,7 +323,7 @@ See Changelog
 
 =head1 AUTHOR
 
-Gabriel Guillon, from Chashew team
+Gabriel Guillon, from Cashew team
 
 korsani-spam@free-spam.fr-spam
 
@@ -313,25 +338,33 @@ Let me know if you have added some features, or removed some bugs ;)
 =cut
 
 package Log::Funlog;
+use Carp;
+use strict;
+use File::Basename;
 
 BEGIN {
 	use Exporter;
+	our ($VERSION, @ISA, @EXPORT, @EXPORT_OK );
 	@ISA=qw(Exporter);
 	@EXPORT=qw( );
-	@EXPORT_OK=qw( &error );
-	$VERSION='0.84';
+	@EXPORT_OK=qw( &error $VERBOSE $LEVELMAX $VERSION );
+	$VERSION='0.85';
+	sub VERSION {
+		(my $me, my $askedver)=@_;
+		$VERSION=~s/(.*)_\d+/$1/;
+		croak "Please update: $me is version $VERSION and you asked version $askedver" if ($VERSION < $askedver);
+	}
 }
-use Carp;
-use strict;
 my @fun;
-eval {require Log::Funlog::Lang};
+our %args;
+
+eval 'use Log::Funlog::Lang 0.3';
 if ($@) {
 	@fun=();
 } else {
-	@fun=Log::Funlog::Lang->new();
+	@fun=@{ (Log::Funlog::Lang->new())[1] };
 }
 #use Sys::Syslog;
-use Scalar::Util qw(tainted);
 my $count=0;
 use vars qw( %args $me $error_header $error $metaheader);
 
@@ -399,19 +432,19 @@ sub new {
 
 
 	# Okay, now sanity checking!
-	# This is cool because we have time, so we can do all sort of checking, calculating, things like that
+	# This is cool because we have time, so we can do all kind of checking, calculating, things like that
 	#########################################
-	if (defined $args{daemon}) {
+	if (defined $args{daemon} and $args{daemon}) {
 		croak 'You want me to be a daemon, but you didn\'t specifie a file to log to...' unless (defined $args{file});
 	}
 	croak "'verbose' option is mandatory." if (! $args{'verbose'});
-	croak "'verbose' should be of the form n/m or max/m" if (($args{'verbose'} !~ /^\d+\/\d+$/) and ($args{'verbose'} !~ /^max\/\d+$/));
+	croak "'verbose' should be of the form n/m or max/m" if (($args{'verbose'} !~ /^\d+\/\d+$/) and ($args{'verbose'} !~ /^[mM][aA][xX]\/\d+$/));
 
 	# Parsing 'verbose' option...
 	#########################################
 	my ($verbose,$levelmax)=split('/',$args{verbose});
 	$levelmax=$levelmax ? $levelmax : "";						#in case it is not defined...
-	$verbose=$levelmax if ($verbose =~ /^max$/);
+	$verbose=$levelmax if ($verbose =~ /^[mM][aA][xX]$/);
 	if (($verbose !~ /\d+/) or ($levelmax !~ /\d+/)) {
 		carp "Arguments in 'verbose' should be of the form n/m, where n and m are numerics.\nAs this is a new feature, I'll assume you didn't upgraded your script so I'll make it compatible...\nAnyhow, consider upgrading soon!\n";
 		croak "No 'levelmax' provided" unless ($args{levelmax});
@@ -430,7 +463,7 @@ sub new {
 	if (defined $args{fun}) {
 		croak "'fun' should only be a number (between 0 and 100, bounds excluded)." if ($args{fun} !~ /^\d+$/);
 		croak "0<fun<=100" if ($args{fun}>100 or $args{fun}<=0);
-		croak "You want fun but Log::Funlog::Lang is not available." if ($#fun <= 0);
+		croak "You want fun but Log::Funlog::Lang is not available, or is too old." if ($#fun <= 0);
 	}
 
 	# Colors
@@ -438,25 +471,25 @@ sub new {
 	#We will build %colors here.
 	#If color is wanted:
 	#	if default is wanted, %colors = %defaultcolors
-	#	if not, %colors = %defaultcolors, overrident by the parameters provided
+	#	if not, %colors = %defaultcolors, overriden by the parameters provided
 	#If no colors is wanted, %colors will be filled with the 'none' colors.
 	#
 	#This way of doing should be quicker :)
 	#
-	if (exists $args{'colors'}) {						#If color is wanted
+	if (exists $args{'colors'} and $args{'colors'} ) {							#If color is wanted
 		use Config;
-		if ($Config{'osname'} eq 'MSWin32') {				#Oh oh! AFAIK MSWin console do not support color...
-			carp 'Colors wanted, but MSwin detected. Colors deactivated.';
+		if ($Config{'osname'} eq 'MSWin32') {				#Oh oh!
+			carp 'Colors wanted, but MSwin detected. Colors deactivated (because not implemented yet)';
 			delete $args{'colors'};
-			$colortable{'none'}='';					#putting 'none' color to void
+			$colortable{'none'}='';							#putting 'none' color to void
 			foreach my $color (keys %defaultcolors) {
 				$colors{$color}=$colortable{'none'};		#and propagating it
 			}
 #			no Config;
-		} else {						#We are not in MSWin...
+		} else {											#We are not in MSWin...
 			if (ref(\$args{'colors'}) eq 'SCALAR') {		#default colors?	
 				%colors=%defaultcolors if ($args{'colors'});
-			} elsif(ref($args{'colors'}) eq 'HASH') {	#No... Overridden colors :)
+			} elsif(ref($args{'colors'}) eq 'HASH') {		#No... Overridden colors :)
 				foreach my $item (keys %defaultcolors) {
 					$colors{$item}=exists ${		#If the color is provided
 						$args{'colors'}
@@ -471,8 +504,8 @@ sub new {
 				croak("'colors' must be type of SCALAR or HASH, not ".ref($args{'colors'})."\n");
 			}
 		}
-	} else {									#no colors? so the color table will contain the color 'none'
-		$colortable{'none'}='' if ($Config{'osname'} eq 'MSWin32');
+	} else {										#no colors? so the color table will contain the color 'none'
+		$colortable{'none'}='';						#Avoid printing "\e[0m" :)
 		foreach my $item (keys %defaultcolors) {
 			$colors{$item}=$colortable{'none'};
 		}
@@ -485,7 +518,11 @@ sub new {
 
 # We define default cosmetic if no one was defined
 #########################################
-	$args{cosmetic}='x' if (not defined $args{cosmetic});
+	if (not defined $args{cosmetic}) {
+		$args{'cosmetic'}='x';
+	} elsif ($args{'cosmetic'} !~ /^[[:^cntrl:]]$/) {
+		croak("'cosmetic' must be one character long, and printable.");
+	}
 
 # Parsing header. Goal is to avoid work in the wr() function
 #########################################
@@ -504,7 +541,7 @@ sub new {
 		$metaheader=replace($metaheader,"d",$colors{'date'}."\$date".$colortable{'none'});
 
 		# but %pp won't vary
-		$me=`basename $0`;
+		$me=basename("$0");
 		chomp $me;
 		$whattoprint{'p'}=1 if ($metaheader=~/\%p.*p/);
 		$metaheader=replace($metaheader,"p",$colors{'prog'}.$me.$colortable{'none'});
@@ -526,7 +563,7 @@ sub new {
 	} else {
 		$handleout=\*STDERR;
 	}
-
+	print $handleout $colortable{'red'}.'** '.$colortable{'white'}.'Log::Funlog Powered!'.$colortable{'red'}." **\n".$colortable{'none'} if (defined $args{'splash'});
 
 	my $self = \&wr;
 	bless $self, $class;			#The function's address is now a Log::Funlog object
@@ -542,6 +579,7 @@ sub wr {
 
 	my $prevhandle=select $handleout;
 
+	my $return_code;
 # Header building!!
 #####################################
 
@@ -605,6 +643,7 @@ sub wr {
 	print $colors{'msg'};
 	while (my $tolog=shift) {			#and then print all the things the user wants me to print
 		print $tolog;
+		$return_code.=$tolog;
 	}
 	print $colortable{'none'};
 	print "\n";
@@ -612,12 +651,12 @@ sub wr {
 	print $fun[1+int(rand $#fun)],"\n" if ($args{fun} and (rand(100)<$args{fun}) and ($count>10));			#write a bit of fun, but not in the first 10 lines
 	select($prevhandle);
 	$count++;
-	return 1;
+	return $return_code;
 }
 sub error {
 	$error=1;
-	wr(1,$error_header," ",@_);
+	my $ec=wr(1,$error_header," ",@_);
 	$error=0;
-	return 1;
+	return $ec;
 }
 1;
